@@ -3,27 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.2 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
+ * Please see the License for the specific language governing rights and 
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
- */
-#include <UserNotification/KUNCUserNotifications.h>
-
+ */ 
 #include <IOKit/IOKitKeys.h>
 #include <IOKit/usb/USBSpec.h>  // USBHub.h should include this file
 #include <IOKit/usb/USBHub.h>
@@ -233,7 +228,7 @@ AppleUSBHubPort::PortInit()
     // non captive devices will come in through the status change handler
     if (!_captive)
     {
-        USBLog(5, "***** AppleUSBHubPort[%p]::PortInit - port %d on hub %p non-captive device - leaving PortInit", this, _portNum, _hub);
+        USBLog(5, "***** AppleUSBHubPort[%p]::PortInit - port %d on hub %p captive device - leaving PortInit", this, _portNum, _hub);
         goto errorExit;
     }
         
@@ -401,7 +396,7 @@ AppleUSBHubPort::SuspendPort( bool suspend)
     USBLog(5, "AppleUSBHubPort[%p]::SuspendPort for port %d", this, _portNum);
 
     do {
-        
+
         // If resuming, need to check that the port was suspended to begin with
         //
         err = _hub->GetPortStatus(&status, _portNum);
@@ -410,10 +405,10 @@ AppleUSBHubPort::SuspendPort( bool suspend)
             IOLog("AppleUSBHubPort::SuspendPort Could not get Port Status: 0x%x\n", err);
             break;
         }
-
+        
         if (!suspend && !(status.statusFlags & kHubPortSuspend) )
         {
-            // We were trying to resume but the port was not supended.  Just ignore the
+            // We were trying to resume but the port was not supended.  Just ignore the 
             // request.
             break;
         }
@@ -421,39 +416,25 @@ AppleUSBHubPort::SuspendPort( bool suspend)
         // OK, set up the handler for the set/clear suspend feature
         //
         SetPortVector(&AppleUSBHubPort::HandleSuspendPortHandler, kHubPortSuspend);
-
+        
         if (suspend)
         {
             err = _hub->SetPortFeature(kUSBHubPortSuspendFeature, _portNum);
             if ( err != kIOReturnSuccess )
             {
                 // Root Hub failed for lucent bug
-                //
+                // 
                 // Callback device with the resume message
-                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort Could not SetPortFeature (%d) (kUSBHubPortSuspendFeature): (0x%x)", this, _portNum, err);
+                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort got error = 0x%x", this, err);
                 if ( _portDevice )
                     _portDevice->message(kIOUSBMessagePortHasBeenResumed, NULL, 0);
-
-            }
+	    }
         }
         else
-        {
             err = _hub->ClearPortFeature(kUSBHubPortSuspendFeature, _portNum);
-            if ( err != kIOReturnSuccess )
-            {
-                USBLog(3, "AppleUSBHubPort[%p]::SuspendPort Could not ClearPortFeature (%d) (kUSBHubPortSuspendFeature): (0x%x)", this, _portNum, err);
-            }
-        }
-
-
+            
+            
     } while (false);
-
-    if ( err != kIOReturnSuccess )
-    {
-        // Set the handler back to default
-        //
-        SetPortVector(&AppleUSBHubPort::DefaultSuspendChangeHandler, kHubPortSuspend);
-    }
     
     return err;
 }
@@ -504,7 +485,7 @@ AppleUSBHubPort::ResetPort()
             err = _hub->ClearPortFeature(kUSBHubPortSuspendFeature, _portNum);
             if (kIOReturnSuccess != err)
             {
-                USBLog(3, "AppleUSBHubPort[%p]::ResetPort Could not ClearPortFeature (%d) (kHubPortSuspend): (0x%x)", this, _portNum, err);
+                IOLog("AppleUSBHubPort::ResetPort Could not ClearPortFeature (%d) (kHubPortSuspend): (0x%x)\n", _portNum, err);
                 break;
             }
         }
@@ -515,19 +496,12 @@ AppleUSBHubPort::ResetPort()
         SetPortVector(&AppleUSBHubPort::HandleResetPortHandler, kHubPortBeingReset);
         
         err = _hub->SetPortFeature(kUSBHubPortResetFeature, _portNum);
-        if (err != kIOReturnSuccess)
-        {
-            USBLog(3, "AppleUSBHubPort[%p]::ResetPort Could not ClearPortFeature (%d) (kUSBHubPortResetFeature): (0x%x)", this, _portNum, err);
-
-            // Return our vector to the default handler
-            //
-            SetPortVector(&AppleUSBHubPort::DefaultResetChangeHandler, kHubPortBeingReset);
+        if(err != kIOReturnSuccess)
             break;
-        }
 
     } while (false);
 
-    if (err == kIOReturnSuccess)
+    if(err == kIOReturnSuccess)
     {
         _bus->WaitForReleaseDeviceZero();
     }
@@ -601,7 +575,7 @@ IOUSBControllerV2 *v2Bus;
  **
  **********************************************************************/
 IOReturn 
-AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     IOReturn		err2 = kIOReturnSuccess;
@@ -615,19 +589,6 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFl
     {
         if (_state != hpsDeadDeviceZero)
         {
-            
-            // Before doing anything, check to see if the device is really there
-            //
-            if ( !(statusFlags & kHubPortConnection) )
-            {
-                // We don't have a connection on this port anymore.
-                //
-                USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - port %d - device has gone away", this, _portNum);
-                _state = hpsDeadDeviceZero;
-                err = kIOReturnNoDevice;
-                break;
-            }
-
             // in the Mac OS 9 state machine (called resetChangeHandler) we skip states 1-3
             // if we are in DeadDeviceZero state (so that we end up setting the address )
             
@@ -777,7 +738,7 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFl
             IOSleep( 2 );
 
             // Release devZero lock
-            USBLog(5, "**5** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler - port %d, Releasing DeviceZero after successful SetAddress to %d", this, _portNum, address);
+            USBLog(5, "**5** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler - port %d, Releasing DeviceZero after successful SetAddress", this, _portNum);
             _bus->ReleaseDeviceZero();
             _devZero = false;
             _state = hpsNormal;
@@ -885,8 +846,7 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFl
         // In MacOS 9, we attempt to get the full DeviceDescriptor at this point, if we had missed it earlier.  On X, we do NOT do this
         // because we would have failed the IOUSBDevice::start if we didn't get it.
         //
-
-        USBLog(5, "**10** AppleUSBHubPort[%p]::AddDeviceResetChangeHandler -  port %d, at addr: %d, Succesful", this, _portNum, address);
+        
         // MacOS STATE 10
         //
         _attachRetry = 0;
@@ -927,8 +887,10 @@ AppleUSBHubPort::AddDeviceResetChangeHandler(UInt16 changeFlags, UInt16 statusFl
     return err;
 }
 
+OSMetaClassDefineReservedUsed(IOUSBController,  13);
+
 IOReturn 
-AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     UInt32		delay = 10;
@@ -1195,7 +1157,7 @@ AppleUSBHubPort::HandleResetPortHandler(UInt16 changeFlags, UInt16 statusFlags)
 }
 
 IOReturn
-AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags)
 {
     USBLog(3, "AppleUSBHubPort[%p]::HandleSuspendPortHandler for port %d, changeFlags: 0x%x", this, _portNum, changeFlags);
     SetPortVector(&AppleUSBHubPort::DefaultSuspendChangeHandler, kHubPortSuspend);
@@ -1208,54 +1170,25 @@ AppleUSBHubPort::HandleSuspendPortHandler(UInt16 changeFlags, UInt16 statusFlags
 }
 
 IOReturn 
-AppleUSBHubPort::DefaultOverCrntChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::DefaultOverCrntChangeHandler(UInt16 changeFlags)
 {
-    IOUSBHubDescriptor		hubDescriptor;
-    bool			individualPortPower = FALSE;
-    UInt16			characteristics;
-    IOUSBHubPortStatus		portStatus;
-    IOReturn			err;
-
-    USBLog(5, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. Port %d", this,  _portNum );
-    err = _hub->GetPortStatus(&portStatus, _portNum);
-
-    if ( err == kIOReturnSuccess )
-    {
-        if ( (portStatus.statusFlags & kHubPortOverCurrent))
-        {
-            USBLog(1, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. OverCurrent condition in Port %d", this,  _portNum );
-            hubDescriptor = _hub->GetCachedHubDescriptor();
-
-            characteristics = USBToHostWord(hubDescriptor.characteristics);
-
-            if ( (characteristics & 0x18) == 0x8 )
-                individualPortPower = TRUE;
-
-            DisplayOverCurrentNotice( individualPortPower );
-        }
-        else
-        {
-            // the OverCurrent status for this port has changed to zero.
-            //
-            USBLog(1, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler. No OverCurrent condition. Ignoring. Port %d", this, _portNum );
-        }
-    }
-    
-    return err;
+    USBLog(3, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler for port %d returning kIOReturnUnsupported", this, _portNum);
+    return kIOReturnUnsupported;
 }
 
 
+
 IOReturn 
-AppleUSBHubPort::DefaultResetChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::DefaultResetChangeHandler(UInt16 changeFlags)
 {
-    USBLog(5, "AppleUSBHubPort[%p]::DefaultResetChangeHandler for port %d returning kIOReturnSuccess", this, _portNum);
+    USBLog(5, "AppleUSBHubPort[%p]::DefaultOverCrntChangeHandler for port %d returning kIOReturnSuccess", this, _portNum);
     return kIOReturnSuccess;
 }
 
 
 
 IOReturn 
-AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags)
 {
     USBLog(5, "AppleUSBHubPort[%p]::DefaultSuspendChangeHandler for port %d returning kIOReturnSuccess", this, _portNum);
     return kIOReturnSuccess;
@@ -1264,7 +1197,7 @@ AppleUSBHubPort::DefaultSuspendChangeHandler(UInt16 changeFlags, UInt16 statusFl
 
 
 IOReturn 
-AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags)
 {
     IOReturn		err = kIOReturnSuccess;
     IOUSBHubPortStatus	status;
@@ -1289,12 +1222,12 @@ AppleUSBHubPort::DefaultEnableChangeHandler(UInt16 changeFlags, UInt16 statusFla
 
 
 IOReturn 
-AppleUSBHubPort::DefaultConnectionChangeHandler(UInt16 changeFlags, UInt16 statusFlags)
+AppleUSBHubPort::DefaultConnectionChangeHandler(UInt16 changeFlags)
 {
     IOReturn	err = kIOReturnSuccess;
     IOUSBHubPortStatus	status;
 
-    USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - handling port %d changes (%x).", this, _portNum, changeFlags);
+    USBLog(5, "AppleUSBHubPort[%p]::DefaultConnectionChangeHandler - handling port %d changes (%d).", this, _portNum, changeFlags);
     _connectionChangedState = 0;
     do
     {
@@ -1493,7 +1426,7 @@ AppleUSBHubPort::PortStatusChangedHandler(void)
         USBLog(5, "AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - status(%8x) - change(%8x) - before call to (%d) handler function", this, _portNum, _portStatus.statusFlags, _portStatus.changeFlags, which);
             
         _statusChangedState = ((which+1) * 20) + 1;
-        err = (this->*_changeHandler[which].handler)(_portStatus.changeFlags, _portStatus.statusFlags);
+        err = (this->*_changeHandler[which].handler)(_portStatus.changeFlags);
         _statusChangedState = ((which+1) * 20) + 2;
         USBLog(5,"AppleUSBHubPort[%p]::PortStatusChangedHandler - port %d - err (%x) on return from  call to (%d) handler function", this, _portNum, err, which);
 
@@ -1523,8 +1456,6 @@ errorExit:
 	
     if ( _devZero )
     {
-        // We should disable the port here as well..
-        //
         _bus->ReleaseDeviceZero();
         _devZero = false;
     }
@@ -1614,7 +1545,7 @@ AppleUSBHubPort::ReleaseDevZeroLock()
     {
         (void) _hub->ClearPortFeature(kUSBHubPortEnableFeature, _portNum);
         
-        USBError(1, "AppleUSBHubPort::ReleaseDevZeroLock()");
+        IOLog("AppleUSBHubPort Releasing devZero lock\n");
         _state = hpsNormal;
  
         if ( _bus )
@@ -1801,14 +1732,3 @@ AppleUSBHubPort::AcquireDeviceZero()
     
     return devZero;
 }
-
-void
-AppleUSBHubPort::DisplayOverCurrentNotice(bool individual)
-{
-    if ( individual )
-        _hub->_device->DisplayUserNotification(kUSBIndividualOverCurrentNotificationType);
-    else
-        _hub->_device->DisplayUserNotification(kUSBGangOverCurrentNotificationType);
-
-    return;
-}
